<?php namespace EwCacheInvalidator;

use EwCacheInvalidator\QueryItem;

/**
 * Class DataStorage
 * @package EwCacheInvalidator
 */
class DataStorage {

    private $table_name = 'ew_cache_invalidation';

    /**
     * DataStorage constructor.
     */
    public function __construct() {

        $this->setupDatabase();
    }

    /**
     * Setup database, create table if it exists.
     */
    private function setupDatabase() {

        if($this->databaseExist()) {
            return;
        }

        global $wpdb;
        $table_name = $wpdb->prefix . $this->table_name;
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE $table_name (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            oc_query TEXT NOT NULL,
            page_id int NOT NULL,
            type VARCHAR(128),
            listener_id VARCHAR(512),
            UNIQUE KEY id (id)
        ) $charset_collate;";

        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    /**
     * Check if database table exists.
     *
     * @return bool
     */
    private function databaseExist() {

        global $wpdb;
        $tablename = $wpdb->prefix . $this->table_name;
        return $wpdb->get_var("SHOW TABLES LIKE '".$tablename."'") !== null;
    }

    /**
     * Save a query item to database.
     *
     * @param \EwCacheInvalidator\QueryItem $item
     * @return bool|int
     */
    public function saveItem(QueryItem $item) {

        global $wpdb;

        if($item->id > 0) {

            $result = $wpdb->query(
                $wpdb->prepare("UPDATE {$wpdb->prefix}{$this->table_name}
                      SET oc_query = %s, page_id = %d, type = %s, listener_id = %s
                      WHERE id = %d", $item->oc_query, $item->page_id, $item->type, $item->listener_id, $item->id)
            );

            if($result !== false) {
                return $item->id;
            }
        }
        else {

            $result = $wpdb->query(
                $wpdb->prepare("INSERT INTO {$wpdb->prefix}{$this->table_name} (oc_query, page_id, type, listener_id)
                      VALUES (%s, %d, %s, %s)", $item->oc_query, $item->page_id, $item->type, $item->listener_id)
            );

            if($result !== false) {
                return $wpdb->insert_id;
            }
        }

        return false;
    }

    /**
     * Get all query items from database.
     *
     * @return array|bool
     */
    public function getAllItems() {

        global $wpdb;
        $result = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}{$this->table_name} ORDER BY page_id");

        if($result !== false) {

            $queryItemArr = [];
            foreach($result as $item) {
                $queryItemArr[] = new QueryItem($item->oc_query, $item->page_id, $item->type, $item->listener_id, $item->id);
            }

            return $queryItemArr;
        }

        return false;
    }

    /**
     * Get query items by page id.
     *
     * @param $page_id
     * @return array|bool
     */
    public function getItemsByPageId($page_id) {

        global $wpdb;
        $result = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}{$this->table_name} WHERE page_id = {$page_id}");

        if($result !== false) {

            $queryItemArr = [];
            foreach($result as $item) {
                $queryItemArr[] = new QueryItem($item->oc_query, $item->page_id, $item->type, $item->listener_id, $item->id);
            }

            return $queryItemArr;
        }

        return false;
    }

    /**
     * Delete all items for given page.
     *
     * @param $page_id
     */
    public function deleteItemsByPageId($page_id) {

        global $wpdb;
        $wpdb->delete($wpdb->prefix . $this->table_name, ['page_id' => $page_id], ['%d']);
    }

    /**
     * Delete item with given id.
     *
     * @param $id
     */
    public function deleteItem($id) {

        global $wpdb;
        $wpdb->delete($wpdb->prefix . $this->table_name, ['id' => $id], ['%d']);
    }

    /**
     * Purge table, remove all items.
     *
     * @return bool
     */
    public function purgeTable() {

        global $wpdb;
        $wpdb->query("TRUNCATE TABLE {$wpdb->prefix}{$this->table_name}");

        return true;
    }
}