<?php namespace EwCacheInvalidator;

use EwCacheInvalidator\PageQueries;
use EwCacheInvalidator\DataStorage;
use EwCacheInvalidator\Lcc;

/**
 * Class PageSettings
 * @package EwCacheInvalidator
 */
class PageSettings {

    private static $query_suffix_option_name = 'ew_cache_invalidator_query_suffix';

    /**
     * PageSettings constructor.
     */
    public function __construct() {

        add_action('admin_enqueue_scripts', function() {

            if(isset($_GET['page']) && $_GET['page'] === 'ew-cache-invalidator') {
                wp_enqueue_script('ew-cache-invalidator-settings-page', plugins_url('../src/assets/js/admin.js', __FILE__));
                wp_register_style('ew-cache-invalidator-admin-style', plugins_url('../src/assets/css/admin.css', __FILE__));
                wp_enqueue_style('ew-cache-invalidator-admin-style' );
            }
        });
    }

    /**
     * Render the admin page.
     */
    public function renderAdminPage() {

        print '<h1>Cache invalidation settings</h1>';

        print '<div class="admin-section">';
            print '<h3>Query suffix</h3>';
            print '<p><textarea type="button" id="set-query-suffix-textarea">' . PageSettings::getQuerySuffix() . '</textarea></p>';
            print '<p><input type="button" value="Set query suffix" class="button button-primary" id="set-query-suffix-btn" /></p>';
            print '<p class="progress-indicator-set-query-suffix progress-indicator"></p>';
            print '<p class="description">Optional, set a query suffix to be appended to all queries registered in LCC. Example: "AND group:Paper1"</p>';
            print '<p class="description">Changes will only apply to queries registered after a change is made.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Run query lookup</h3>';
            print '<p><input type="button" value="Find all queries" class="button button-primary" id="reset-all-queries-btn" /></p>';
            print '<p class="progress-indicator-reset-all-queries progress-indicator"></p>';
            print '<p class="description">This action removes all existing queries before adding.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Delete all queries</h3>';
            print '<p><input type="button" value="Delete queries" class="button button-primary" id="delete-all-queries-btn" /></p>';
            print '<p class="progress-indicator-delete-all-queries progress-indicator"></p>';
            print '<p class="description">Delete all saved queries, listeners will be deregistered as well.</p>';
        print '</div>';

        print '<div class="admin-section">';
            print '<h3>Invalidation endpoint status</h3>';
            print '<p class="ew-cache-invalidator-endpoint-status">Checking endpoint status.. <span class="ew-cache-spinner"></span></p>';
            print '<p class="description">For this plugin to work, the environment has to support the <strong>"/ledge-invalidation"</strong> path.</p>';
            print '<p class="description">If it does not work, please look up plugin instructions on how to enable this.</p>';
        print '</div>';
    }

    /**
     * Save query suffix via ajax.
     */
    public static function ajaxSaveQuerySuffix() {

        $status = false;
        if(isset($_POST['suffix'])) {
            $status = self::saveQuerySuffix($_POST['suffix']);
        }

        wp_send_json([
            'status' =>  $status
        ]);

        die();
    }

    /**
     * Reset all queries via ajax.
     */
    public static function ajaxResetAllQueries() {

        wp_send_json([
            'status' =>  self::resetAllQueries()
        ]);

        die();
    }

    /**
     * Delete all queries via ajax.
     */
    public static function ajaxDeleteAllQueries() {

        $dataStorage = new DataStorage();
        wp_send_json([
            'status' => $dataStorage->purgeTable()
        ]);

        die();
    }

    /**
     * Check invalidation endpoint status via ajax.
     */
    public static function ajaxCheckEndpointStatus() {

        wp_send_json([
            'status' => self::checkEndpointStatus()
        ]);

        die();
    }

    /**
     * Get query suffix.
     *
     * @return mixed
     */
    public static function getQuerySuffix() {
        return get_option(self::$query_suffix_option_name, '');
    }
    
    private static function saveQuerySuffix($suffix) {
        return update_option(self::$query_suffix_option_name, $suffix);
    }

    /**
     * Reset all queries.
     *
     * @return bool
     */
    private static function resetAllQueries() {

        $pageQueries = new PageQueries();
        $dataStorage = new DataStorage();
        $lcc = new Lcc();

        $success = $dataStorage->purgeTable();
        // TODO: Deregister all listeners as well.
        if($success) {

            $queryArr = $pageQueries->getAllQueries();

            foreach($queryArr as $query) {

                if(is_array($query) && count($query) >= 3) {

                    // TODO: Move this out to own function.
                    $listener_id = $lcc->registerListener($query[0]);
                    if($listener_id !== false) {
                        $dataStorage->saveItem(new QueryItem($query[0], $query[1], $query[2], $listener_id));
                    }
                }
            }

            return true;
        }

        return false;
    }

    /**
     * Get cache invalidation endpoint status.
     *
     * @return bool
     */
    public static function checkEndpointStatus() {

        $response = wp_remote_post(get_site_url() . '/ledge-invalidate/', [
            'user-agent' => 'ew-ledge-invalidation',
            'sslverify' => false,
            'body' => [
                'testing' => 'true'
            ]
        ]);

        if(!is_wp_error($response) && $response['response']['code'] === 200) {
            return true;
        }

        return false;
    }

    /**
     * Check if we should listen to events.
     *
     * @return bool
     */
    public static function checkQueryEventsActive() {
        return true;
    }
}